struct conteur_struct  
{
  float angle;
  float angle2;
  float v_angle2;

  float dt;

  unsigned int blur_mode;

 
  int fps;

  unsigned int courbe;
  unsigned int mix_reprise;
  unsigned int last_flash;
  unsigned int draw_mode;
  unsigned int burn_mode;
  unsigned int k1;
  unsigned int k2;
  unsigned int k3;
 
  
  unsigned int general;
  int term_display;
  int fullscreen;
  int psy;
  int analyser;
  int freeze ;
  int freeze_mode ;

  
  int triplet;
};

