int nice(int inc);  
void *renderer(void);
void fade(float variable, unsigned char * dim);
void draw_mode(int mode);
void on_beat(int beat);
void on_reprise(void);
void copy_and_fade(float factor);
void render_deformation(int defmode);
void render_blur(int blur);
void manage_dynamic_and_states_open(void);
void manage_states_close(void);
